
import React, { lazy, Suspense } from 'react';
import { createRoot } from 'react-dom/client';
import './index.css';
import ErrorBoundary from './components/ErrorBoundary';

// Функция для создания предварительных подключений к критически важным доменам
function createPreconnects() {
  try {
    const domains = [
      'cdn.gpteng.co', 
      'images.unsplash.com', 
      'fonts.googleapis.com', 
      'fonts.gstatic.com'
    ];
    
    domains.forEach(domain => {
      // Создаем preconnect для установления раннего соединения
      if (!document.querySelector(`link[rel="preconnect"][href="https://${domain}"]`)) {
        const preconnect = document.createElement('link');
        preconnect.rel = 'preconnect';
        preconnect.href = `https://${domain}`;
        preconnect.crossOrigin = 'anonymous';
        document.head.appendChild(preconnect);
      }
      
      // Создаем DNS prefetch как запасной вариант
      if (!document.querySelector(`link[rel="dns-prefetch"][href="https://${domain}"]`)) {
        const dnsPrefetch = document.createElement('link');
        dnsPrefetch.rel = 'dns-prefetch';
        dnsPrefetch.href = `https://${domain}`;
        document.head.appendChild(dnsPrefetch);
      }
    });
  } catch (error) {
    console.error("Ошибка при создании предварительных подключений:", error);
  }
}

// Запускаем оптимизации один раз перед рендерингом
(function runOptimizations() {
  try {
    createPreconnects();
  } catch (error) {
    console.error("Ошибка при выполнении оптимизаций перед рендерингом:", error);
  }
})();

// Ленивая загрузка основного компонента App
const App = lazy(() => import('./App'));

// Ищем корневой контейнер
const container = document.getElementById('root');

if (container) {
  try {
    // Используем createRoot из React 18 для concurrent mode
    const root = createRoot(container);
    
    // Добавляем класс для включения аппаратного ускорения для более плавных анимаций
    document.documentElement.classList.add('hardware-accelerated');
    
    // Рендерим приложение с оптимизированной загрузкой
    root.render(
      <React.StrictMode>
        <ErrorBoundary 
          fallback={
            <div className="flex flex-col items-center justify-center min-h-screen p-8 text-center bg-neiro-dark text-white">
              <h2 className="text-2xl font-bold text-red-400 mb-4">Произошла ошибка. Пожалуйста, обновите страницу.</h2>
              <p className="text-neiro-gray-light mb-6">Мы уже работаем над исправлением проблемы.</p>
              <button 
                onClick={() => window.location.reload()} 
                className="px-6 py-3 bg-gradient-to-r from-neiro-primary to-neiro-secondary text-white rounded-md hover:opacity-90 transition-opacity"
              >
                Обновить страницу
              </button>
            </div>
          }
          onError={(error, errorInfo) => {
            console.error("Ошибка корневого уровня:", error, errorInfo);
          }}
        >
          <Suspense fallback={
            <div className="flex items-center justify-center h-screen bg-neiro-dark">
              <div className="flex flex-col items-center">
                <div className="w-16 h-16 border-4 border-neiro-primary border-t-transparent rounded-full animate-spin"></div>
                <p className="mt-4 text-neiro-primary animate-pulse">Загрузка приложения...</p>
              </div>
            </div>
          }>
            <App />
          </Suspense>
        </ErrorBoundary>
      </React.StrictMode>
    );
  } catch (error) {
    console.error('Ошибка при рендеринге приложения:', error);
    
    // Показываем запасной UI в случае критической ошибки
    container.innerHTML = `
      <div style="display: flex; flex-direction: column; align-items: center; justify-content: center; min-height: 100vh; padding: 2rem; text-align: center; background-color: #121212; color: white;">
        <h2 style="font-size: 1.5rem; font-weight: bold; color: #f87171; margin-bottom: 1rem;">Произошла критическая ошибка. Пожалуйста, обновите страницу.</h2>
        <p style="color: #9ca3af; margin-bottom: 1.5rem;">Мы уже работаем над исправлением проблемы.</p>
        <button 
          onclick="window.location.reload()" 
          style="padding: 0.75rem 1.5rem; background-image: linear-gradient(to right, #3b82f6, #6366f1); color: white; border-radius: 0.375rem; border: none; cursor: pointer;"
        >
          Обновить страницу
        </button>
      </div>
    `;
  }
} else {
  console.error('Не удалось найти корневой элемент');
  
  // Добавляем аварийный UI, если даже root-элемент не найден
  document.body.innerHTML = `
    <div style="display: flex; flex-direction: column; align-items: center; justify-content: center; min-height: 100vh; padding: 2rem; text-align: center; background-color: #121212; color: white;">
      <h2 style="font-size: 1.5rem; font-weight: bold; color: #f87171; margin-bottom: 1rem;">Не удалось загрузить приложение.</h2>
      <p style="color: #9ca3af; margin-bottom: 1.5rem;">Пожалуйста, обновите страницу или попробуйте позже.</p>
      <button 
        onclick="window.location.reload()" 
        style="padding: 0.75rem 1.5rem; background-image: linear-gradient(to right, #3b82f6, #6366f1); color: white; border-radius: 0.375rem; border: none; cursor: pointer;"
      >
        Обновить страницу
      </button>
    </div>
  `;
}
